/*****************************************************************************************************************************
--
--    File Name    : Bayer_Conversion_Top.v 

--    Description  : This module implements Bayer_Conversion.

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module BayerConversionTop #(parameter g_DATAWIDTH = 8,
                            parameter g_X_RES_WIDTH = 11,
                            parameter g_DISPLAY_RESOLUTION = 1280,
                            parameter g_VERT_DISPLAY_RESOLUTION = 720)
                          (input wire SYS_CLK_I,
                           input wire RESET_n_I,
                           input wire[g_DATAWIDTH - 1 : 0] Data_In_i,
                           input wire Data_In_Vld_i,
                           output wire[(g_DATAWIDTH) - 1 : 0] R_Out_o,
                           output wire[(g_DATAWIDTH) - 1 : 0] G_Out_o,
                           output wire[(g_DATAWIDTH) - 1 : 0] B_Out_o,
                           output wire RGB_Vld_out_o);
  wire[(g_DATAWIDTH * 3) - 1 : 0] O;
  wire[(g_DATAWIDTH) - 1 : 0] I;
  wire[(g_DATAWIDTH) - 1 : 0] l;
  wire[(g_DATAWIDTH) - 1 : 0] o;
  wire i;
  CFA_RGB_Decoder #(.g_DATAWIDTH(g_DATAWIDTH), .g_X_RES_WIDTH(g_X_RES_WIDTH), .g_DISPLAY_RESOLUTION(g_DISPLAY_RESOLUTION), .g_VERT_DISPLAY_RESOLUTION(g_VERT_DISPLAY_RESOLUTION)) OI(.SYS_CLK_I(SYS_CLK_I),
                                                                                                                                                                                     .RESET_n_I(RESET_n_I),
                                                                                                                                                                                     .Data_In_i(Data_In_i),
                                                                                                                                                                                     .Data_In_Vld_i(Data_In_Vld_i),
                                                                                                                                                                                     .R_Out_o(I),
                                                                                                                                                                                     .G_Out_o(l),
                                                                                                                                                                                     .B_Out_o(o),
                                                                                                                                                                                     .RGB_Vld_out_o(i));
  assign O = { I, l, o };
  FM_Median_Filter #(.g_DATAWIDTH(g_DATAWIDTH * 3), .g_X_RES_WIDTH(g_X_RES_WIDTH), .g_DISPLAY_RESOLUTION(g_DISPLAY_RESOLUTION), .g_VERT_DISPLAY_RESOLUTION(g_VERT_DISPLAY_RESOLUTION)) II(.SYS_CLK_I(SYS_CLK_I),
                                                                                                                                                                                          .RESET_n_I(RESET_n_I),
                                                                                                                                                                                          .Data_In_i(O),
                                                                                                                                                                                          .Data_In_Vld_i(i),
                                                                                                                                                                                          .R_Out_o(R_Out_o),
                                                                                                                                                                                          .G_Out_o(G_Out_o),
                                                                                                                                                                                          .B_Out_o(B_Out_o),
                                                                                                                                                                                          .RGB_Vld_out_o(RGB_Vld_out_o));
endmodule
